@@ -17,9 +17,12 @@ import functools
 
 import tensorflow as tf  # pylint: disable=unused-import
 from tensorflow_examples.lite.model_maker.core import compat
+from tensorflow_examples.lite.model_maker.core.api import mm_export
+from tensorflow_examples.lite.model_maker.core.task.model_spec import util
 from tensorflow_examples.lite.model_maker.third_party.recommendation.ml.model import recommendation_model as _rm
 
 
+@mm_export('recommendation.ModelSpec')
 class RecommendationSpec(object):
   """Recommendation model spec."""
 
@@ -101,10 +104,34 @@ class RecommendationSpec(object):
     """
     return _rm.RecommendationModel(self.params)
 
+  def get_default_quantization_config(self):
+    """Gets the default quantization configuration."""
+    return None
+
 
 recommendation_bow_spec = functools.partial(
     RecommendationSpec, encoder_type='bow')
+recommendation_bow_spec.__doc__ = util.wrap_doc(
+    RecommendationSpec,
+    'Creates Recommendation Bag-of-Word (BoW) model spec. See also: `tflite_model_maker.recommendation.ModelSpec`.'
+)
+mm_export('recommendation.BowSpec').export_constant(__name__,
+                                                    'recommendation_bow_spec')
+
 recommendation_cnn_spec = functools.partial(
     RecommendationSpec, encoder_type='cnn')
+recommendation_cnn_spec.__doc__ = util.wrap_doc(
+    RecommendationSpec,
+    'Creates Recommendation CNN model spec. See also: `tflite_model_maker.recommendation.ModelSpec`.'
+)
+mm_export('recommendation.CnnSpec').export_constant(__name__,
+                                                    'recommendation_cnn_spec')
+
 recommendation_rnn_spec = functools.partial(
     RecommendationSpec, encoder_type='rnn')
+recommendation_rnn_spec.__doc__ = util.wrap_doc(
+    RecommendationSpec,
+    'Creates Recommendation RNN model spec. See also: `tflite_model_maker.recommendation.ModelSpec`.'
+)
+mm_export('recommendation.RnnSpec').export_constant(__name__,
+                                                    'recommendation_rnn_spec')
