@@ -11,7 +11,11 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-"""APIs to train an on-device recommendation model."""
+"""APIs to train an on-device recommendation model.
+
+Demo code:
+https://github.com/tensorflow/examples/blob/master/tensorflow_examples/lite/model_maker/demo/recommendation_demo.py
+"""
 
 from tensorflow_examples.lite.model_maker.core.data_util.recommendation_dataloader import RecommendationDataLoader as DataLoader
 from tensorflow_examples.lite.model_maker.core.task.model_spec.recommendation_spec import recommendation_bow_spec as BowSpec
